/*
 * binclock.c
 *
 * Released under GPL
 *
 * Copyright (C) 2004 Nico Golde <nico@ngolde.de>
 * Homepage: http://www.ngolde.de
 * Latest change: Die Feb 10 21:24:20 CET 2004
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Description:
 * binary clock for text terminals
 * with color support
*/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <getopt.h>

#define MAX 4096

/* version number */
static char *versionstring="BinClock 1.2.2 [2004-02-10]";

/* help */
static int bclock_usage(void){
    puts (versionstring);
    puts("usage: binclock [options]\n"
       "\n"
       "short options:\n"
       "-v      print the version number + release date + exit.\n"
       "-h      print version info and startup options  + exit.\n"
       "-l      loop the time every second.\n\n"
       "long options:\n"
       "--color=[on|off] default: on\n"
        );
 exit(0); 
}

/* show version */
static int bclock_version(void){
    puts (versionstring);
    exit(0);
}

/* options */
static struct option long_options[] = {
   {"help", 0, NULL, 'h'},
   {"version", 0, NULL, 'v'},
   {"color", 1, NULL, 'c'},
   {"loop", 0, NULL, 'l'},
   {NULL,0,NULL,0}
};
typedef struct{
    char *color_one;
    char *color_zero;
} OPTIONS;
OPTIONS o;

/* ignore whitespace etc. */
void dewhite(char *s) {
    while (*s) {
        if (*s==' ' || *s=='\t' || *s=='-' || *s=='/') strcpy(s, s+1);
        else s++;
    }
}

static int parse_config(void){
    FILE *conf;
    char buffer[MAX],*p, *home;
    unsigned int line=1;
    char *filename = "/.binclockrc";

    /* get the path of personal config file */
    if((home=getenv("HOME")))
        strdup(home);
        strncat(home, filename, sizeof(home) - strlen(filename) - 1);

    /* try to open global config file */
    if((conf=fopen(home,"r"))==NULL){
        if((conf=fopen("/etc/binclockrc","r"))==NULL){
            if((conf=fopen("/usr/local/etc/binclockrc","r"))==NULL){    
                printf("binclockrc: no file found ..\n");
                
                /* if there is no config file */
                o.color_one="red";
                o.color_zero="white";
		        return -1;
            }
        }
    }
    
    /* parse config file */
    while(fgets(buffer, MAX, conf)!=NULL){
        line++;

        /* ignore comments */
         for (p=buffer; *p && *p!='#'; p++); *p='\0';
         for (p=buffer; *p && *p!='='; p++);
         if (!*p) continue;
         *(p++)='\0';
         dewhite(buffer);
         if(!strncasecmp(buffer,"color_one",9))   o.color_one=strdup(p);
         else if(!strncasecmp(buffer,"color_zero",10)) o.color_zero=strdup(p);
         else{
            printf("parse error in:\nline: %d\nstring: %s\n\n",line,buffer);
            exit(1);
         }
    }
    fclose(conf);
return 0;    
}    


/* main function for showing time and filling the matrix */
static void bin_time(int color){

    /* colors */
    const char *const red = "\033[40;31;1m";
    const char *const white = "\033[40;37;1m";
    const char *const green = "\033[40;32;1m";
    const char *const yellow = "\033[40;33;1m";
    const char *const blue = "\033[40;34;1m";
    const char *const magenta = "\033[40;35;1m";
    const char *const cyan = "\033[40;36;1m";
    int i,j;
    int clock[10][10];
    time_t t;
    char tmpstr[20];
    struct tm *tmworld;
    
    t=time(0);
    
    /* fetch localtime */
    tmworld=localtime(&t);
    
    /* fill the tmpstr with hours, minutes and seconds */
    snprintf(tmpstr, 1024,"%02i%02i%02i",
             tmworld->tm_hour, tmworld->tm_min, tmworld->tm_sec);
    
    /* fill the matrix */
    for(i=0; i<6;i++){
        clock[i][0] = 0;
        clock[i][1] = 0;
        clock[i][2] = 0;
        clock[i][3] = 0;
        switch(tmpstr[i]){
            case'1': 
                clock[i][3]=1;
                    break;
            case'2':
                clock[i][2]=1;
                    break;
            case'3':
                clock[i][3]=1;
                clock[i][2]=1;
                    break;
            case'4':
                clock[i][1]=1;
                    break;
            case'5':
                clock[i][3]=1;
                clock[i][1]=1;
                    break;
            case'6':
                clock[i][2]=1;
                clock[i][1]=1;
                    break;
            case'7':
                clock[i][3]=1;
                clock[i][2]=1;
                clock[i][1]=1;
                    break;
            case'8':
                clock[i][0]=1;
                    break;
            case'9':
                clock[i][3]=1;
                clock[i][0]=1;
                    break;
            }
    }        
    
    /* print the content of the matrix */
    for(i=0;i<4;i++){
        for(j=0;j<6;j++){

            /* if binclock is started with --color=on option */
            if(color){
                if(clock[j][i]==1){
                
                    /* choose colors */
                    if(strstr(o.color_one,"white"))
                        printf("%s%s ",white,"1");
                    else if (strstr(o.color_one,"red"))
                        printf("%s%s ", red, "1");
                    else if (strstr(o.color_one, "green"))
                        printf("%s%s ", green, "1");
                    else if (strstr(o.color_one, "yellow"))
                        printf("%s%s ",yellow, "1");
                    else if (strstr(o.color_one, "blue"))
                        printf("%s%s ",blue, "1");
                    else if (strstr(o.color_one, "magenta"))
                        printf("%s%s ",magenta, "1");
                    else if (strstr(o.color_one,"cyan"))
                        printf("%s%s ",cyan,"1");
                    else{
                        printf("false color in config file: %s\n\n", o.color_one);
                        exit(1);
                    }
                }
                else{
                    
                    /* choose colors */
                    if(strstr(o.color_zero,"white"))
                        printf("%s%s ",white,"0");
                    else if (strstr(o.color_zero,"red"))
                        printf("%s%s ", red, "0");
                    else if (strstr(o.color_zero, "green"))
                        printf("%s%s ", green, "0");
                    else if (strstr(o.color_zero, "yellow"))
                        printf("%s%s ",yellow, "0");
                    else if (strstr(o.color_zero, "blue"))
                        printf("%s%s ",blue, "0");
                    else if (strstr(o.color_zero, "magenta"))
                        printf("%s%s ",magenta, "0");
                    else if (strstr(o.color_zero,"cyan"))
                        printf("%s%s ",cyan,"0");
                    else{
                        printf("false color in config file: %s\n\n",o.color_zero);
                        exit(1);
                    }    
                }   
            }

            /* print time without color */
            else{
                if(clock[j][i])
                    printf("1 ");
                else 
                    printf("0 ");
            }
        }    
        printf("\n");
    }       

    if(color){

        /* set the terminal color to white */
        printf("\033[0m");
    }
    
    /* flush the standard output */
    fflush(stdout);
}    

/* main function */
int main(int argc, char *argv[]){
    int l=0,c=1, option,akt_optind,option_index=0;

    /* parse program arguments */
    while(1){
        akt_optind=optind;
        option=getopt_long(argc,argv,"lc:hv",long_options,&option_index);
        
        /* if there is o argument leave while */
        if(option==EOF)
            break;
        switch(option){
            
            /* call the help function */
            case 'h':
                bclock_usage();
                break;

            /* call the version function */    
            case 'v':
                bclock_version();
                break;

            /* if -l option is set */
            case 'l':
                l=1;
                    break;    

            /* if --color option is set */
            case 'c':
                if(!strcmp(optarg,"on")){
                    c=1;
                }
                else if (!strcmp(optarg,"off")){
                    c=0;
                }

                /* if --color option is not on or off */
                else{
                    puts(versionstring);
                    printf("Unknown option: \"--%s\"\n", optarg);
                    printf("More info with: \"%s -h\"\n",argv[0]);
                    return -1;
                        break;
                }
                    break;
            default:
                bclock_usage();                
                break;
        }        
    }    
    parse_config();
    do{ 
        
        /* clear the screen */
        system("clear");
        bin_time(c);
        
        /* sleep one second */
        sleep(l);
    } while(l);    

return 0;
}
